@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -18,21 +18,21 @@
 """
 This module implements the interface with the command line and the logger.
 """
+
 import logging
 import os
 import sys
-
-from argh import ArghParser, named, arg, expects_obj
 from argparse import SUPPRESS, ArgumentTypeError
 
+from argh import ArghParser, arg, expects_obj, named
+
+import barman.config
+import barman.diagnose
 from barman import output
 from barman.infofile import BackupInfo
-from barman import lockfile
 from barman.server import Server
-import barman.diagnose
-import barman.config
-from barman.utils import drop_privileges, configure_logging, parse_log_level
-
+from barman.utils import configure_logging, drop_privileges, parse_log_level
+from barman.xlog import BadXlogSegmentName
 
 _logger = logging.getLogger(__name__)
 
@@ -53,36 +53,51 @@ def check_positive(value):
         raise ArgumentTypeError("'%s' is not a valid positive integer" % value)
     return int_value
 
+
 @named('list-server')
 @arg('--minimal', help='machine readable output')
 def list_server(minimal=False):
     """
     List available servers, with useful information
     """
+    # Get every server, both inactive and temporarily disabled
     servers = get_server_list()
     for name in sorted(servers):
         server = servers[name]
+
+        # Exception: manage_server_command is not invoked here
+        # Normally you would call manage_server_command to check if the
+        # server is None and to report inactive and disabled servers, but here
+        # we want all servers and the server cannot be None
+
         output.init('list_server', name, minimal=minimal)
-        output.result('list_server', name, server.config.description)
+        description = server.config.description
+        # If the server has been manually disabled
+        if not server.config.active:
+            description += " (inactive)"
+        # If server has configuration errors
+        elif server.config.disabled:
+            description += " (WARNING: disabled)"
+        output.result('list_server', name, description)
     output.close_and_exit()
 
 
 def cron():
     """
-    Run maintenance tasks
+    Run maintenance tasks (global command)
     """
-    lockname = os.path.join(barman.__config__.barman_home, '.cron.lock')
-    try:
-        with lockfile.LockFile(lockname, raise_if_fail=True):
-            servers = [Server(conf) for conf in barman.__config__.servers()]
-            for server in servers:
-                server.cron()
-    except lockfile.LockFileBusy:
-        output.info("Another cron is running")
+    # Skip inactive and temporarily disabled servers
+    servers = get_server_list(skip_inactive=True, skip_disabled=True)
+    for name in sorted(servers):
+        server = servers[name]
+
+        # Exception: manage_server_command is not invoked here
+        # Normally you would call manage_server_command to check if the
+        # server is None and to report inactive and disabled servers,
+        # but here we have only active and well configured servers.
+
+        server.cron()
 
-    except lockfile.LockFilePermissionDenied:
-        output.error("Permission denied, unable to access '%s'",
-                     lockname)
     output.close_and_exit()
 
 
@@ -109,15 +124,14 @@ def server_completer_all(prefix, parsed_args, **kwargs):
 def backup_completer(prefix, parsed_args, **kwargs):
     global_config(parsed_args)
     server = get_server(parsed_args)
-    backup_name = getattr(parsed_args, 'backup_id', None) or ''
-    if server:
-        backups = server.get_available_backups()
-        for backup_id in sorted(backups, reverse=True):
-            if backup_id.startswith(prefix):
-                yield backup_id
-        for special_id in ('latest', 'last', 'oldest', 'first'):
-            if len(backups) > 0 and special_id.startswith(prefix):
-                yield special_id
+
+    backups = server.get_available_backups()
+    for backup_id in sorted(backups, reverse=True):
+        if backup_id.startswith(prefix):
+            yield backup_id
+    for special_id in ('latest', 'last', 'oldest', 'first'):
+        if len(backups) > 0 and special_id.startswith(prefix):
+            yield special_id
 
 
 @arg('server_name', nargs='+',
@@ -150,14 +164,16 @@ def backup_completer(prefix, parsed_args, **kwargs):
 @expects_obj
 def backup(args):
     """
-    Perform a full backup for the given server
+    Perform a full backup for the given server (supports 'all')
     """
-    servers = get_server_list(args)
+    servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
         server = servers[name]
-        if server is None:
-            output.error("Unknown server '%s'" % name)
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
             continue
+
         if args.reuse_backup is not None:
             server.config.reuse_backup = args.reuse_backup
         if args.retry_sleep is not None:
@@ -181,13 +197,15 @@ def list_backup(args):
     """
     List available backups for the given server (supports 'all')
     """
-    servers = get_server_list(args)
+    servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
         server = servers[name]
-        output.init('list_backup', name, minimal=args.minimal)
-        if server is None:
-            output.error("Unknown server '%s'" % name)
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
             continue
+
+        output.init('list_backup', name, minimal=args.minimal)
         server.list_backups()
     output.close_and_exit()
 
@@ -200,12 +218,14 @@ def status(args):
     """
     Shows live information and status of the PostgreSQL server
     """
-    servers = get_server_list(args)
+    servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
         server = servers[name]
-        if server is None:
-            output.error("Unknown server '%s'" % name)
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
             continue
+
         output.init('status', name)
         server.status()
     output.close_and_exit()
@@ -219,12 +239,14 @@ def rebuild_xlogdb(args):
     """
     Rebuild the WAL file database guessing it from the disk content.
     """
-    servers = get_server_list(args)
+    servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
         server = servers[name]
-        if server is None:
-            output.error("Unknown server '%s'", name)
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
             continue
+
         server.rebuild_xlogdb()
     output.close_and_exit()
 
@@ -234,10 +256,12 @@ def rebuild_xlogdb(args):
      help='specifies the server name for the command')
 @arg('--target-tli', help='target timeline', type=int)
 @arg('--target-time',
-     help='target time. You can use any valid unambiguous representation. e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
+     help='target time. You can use any valid unambiguous representation. '
+          'e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
 @arg('--target-xid', help='target transaction ID')
 @arg('--target-name',
-     help='target name created previously with pg_create_restore_point() function call')
+     help='target name created previously with '
+          'pg_create_restore_point() function call')
 @arg('--exclusive',
      help='set target xid to be non inclusive', action="store_true")
 @arg('--tablespace',
@@ -245,9 +269,11 @@ def rebuild_xlogdb(args):
      metavar='NAME:LOCATION', action='append')
 @arg('--remote-ssh-command',
      metavar='SSH_COMMAND',
-     help='This options activates remote recovery, by specifying the secure shell command '
-          'to be launched on a remote host. It is the equivalent of the "ssh_command" server'
-          'option in the configuration file for remote recovery. Example: "ssh postgres@db2"')
+     help='This options activates remote recovery, by specifying the secure '
+          'shell command to be launched on a remote host. It is '
+          'the equivalent of the "ssh_command" server option in '
+          'the configuration file for remote recovery. '
+          'Example: "ssh postgres@db2"')
 @arg('backup_id',
      completer=backup_completer,
      help='specifies the backup ID to recover')
@@ -267,14 +293,14 @@ def recover(args):
     Recover a server at a given time or xid
     """
     server = get_server(args)
-    if server is None:
-        output.error("Unknown server '%s'", args.server_name)
-        output.close_and_exit()
+
     # Retrieves the backup
-    backup = parse_backup_id(server, args)
-    if backup is None or backup.status != BackupInfo.DONE:
-        output.error("Unknown backup '%s' for server '%s'",
-                     args.backup_id, args.server_name)
+    backup_id = parse_backup_id(server, args)
+    if backup_id.status != BackupInfo.DONE:
+        output.error(
+            "Cannot recover from backup '%s' of server '%s': "
+            "backup status is not DONE",
+            args.backup_id, server.config.name)
         output.close_and_exit()
 
     # decode the tablespace relocation rules
@@ -284,14 +310,17 @@ def recover(args):
             try:
                 tablespaces.update([rule.split(':', 1)])
             except ValueError:
-                output.error("Invalid tablespace relocation rule '%s'\n"
+                output.error(
+                    "Invalid tablespace relocation rule '%s'\n"
                     "HINT: The valid syntax for a relocation rule is "
                     "NAME:LOCATION", rule)
                 output.close_and_exit()
 
     # validate the rules against the tablespace list
-    valid_tablespaces = [tablespace_data.name for tablespace_data in
-                         backup.tablespaces] if backup.tablespaces else []
+    valid_tablespaces = []
+    if backup_id.tablespaces:
+        valid_tablespaces = [tablespace_data.name for tablespace_data in
+                             backup_id.tablespaces]
     for item in tablespaces:
         if item not in valid_tablespaces:
             output.error("Invalid tablespace name '%s'\n"
@@ -312,7 +341,7 @@ def recover(args):
         server.config.basebackup_retry_sleep = args.retry_sleep
     if args.retry_times is not None:
         server.config.basebackup_retry_times = args.retry_times
-    server.recover(backup,
+    server.recover(backup_id,
                    args.destination_directory,
                    tablespaces=tablespaces,
                    target_tli=args.target_tli,
@@ -328,18 +357,21 @@ def recover(args):
 @named('show-server')
 @arg('server_name', nargs='+',
      completer=server_completer_all,
-     help="specifies the server names to show ('all' will show all available servers)")
+     help="specifies the server names to show "
+          "('all' will show all available servers)")
 @expects_obj
 def show_server(args):
     """
     Show all configuration parameters for the specified servers
     """
-    servers = get_server_list(args)
+    servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
         server = servers[name]
-        if server is None:
-            output.error("Unknown server '%s'" % name)
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
             continue
+
         output.init('show_server', name)
         server.show()
     output.close_and_exit()
@@ -363,22 +395,36 @@ def check(args):
     servers = get_server_list(args)
     for name in sorted(servers):
         server = servers[name]
-        if server is None:
-            output.error("Unknown server '%s'" % name)
+
+        # Validate the returned server
+        if not manage_server_command(
+                server, name, skip_inactive=False,
+                skip_disabled=False, disabled_is_error=False):
             continue
-        output.init('check', name)
+
+        # If the server has been manually disabled
+        if not server.config.active:
+            name += " (inactive)"
+        # If server has configuration errors
+        elif server.config.disabled:
+            name += " (WARNING: disabled)"
+        output.init('check', name, server.config.active)
         server.check()
     output.close_and_exit()
 
-@expects_obj
-def diagnose(args):
+
+def diagnose():
     """
     Diagnostic command (for support and problems detection purpose)
     """
-    servers = get_server_list(None)
-    barman.diagnose.exec_diagnose(servers)
+    # Get every server (both inactive and temporarily disabled)
+    servers = get_server_list(on_error_stop=False, suppress_error=True)
+    # errors list with duplicate paths between servers
+    errors_list = barman.__config__.servers_msg_list
+    barman.diagnose.exec_diagnose(servers, errors_list)
     output.close_and_exit()
 
+
 @named('show-backup')
 @arg('server_name',
      completer=server_completer,
@@ -392,16 +438,10 @@ def show_backup(args):
     This method shows a single backup information
     """
     server = get_server(args)
-    if server is None:
-        output.error("Unknown server '%s'" % args.server_name)
-    else:
-        # Retrieves the backup
-        backup_info = parse_backup_id(server, args)
-        if backup_info is None:
-            output.error("Unknown backup '%s' for server '%s'" % (
-                args.backup_id, args.server_name))
-        else:
-            server.show_backup(backup_info)
+
+    # Retrieves the backup
+    backup_info = parse_backup_id(server, args)
+    server.show_backup(backup_info)
     output.close_and_exit()
 
 
@@ -415,28 +455,30 @@ def show_backup(args):
 @arg('--target', choices=('standalone', 'data', 'wal', 'full'),
      default='standalone',
      help='''
-     Possible values are: data (just the data files), standalone (base backup files, including required WAL files),
-     wal (just WAL files between the beginning of base backup and the following one (if any) or the end of the log) and
-     full (same as data + wal). Defaults to %(default)s
-     '''
-)
+         Possible values are: data (just the data files), standalone
+         (base backup files, including required WAL files),
+         wal (just WAL files between the beginning of base
+         backup and the following one (if any) or the end of the log) and
+         full (same as data + wal). Defaults to %(default)s''')
 @expects_obj
 def list_files(args):
-    """ List all the files for a single backup
+    """
+    List all the files for a single backup
     """
     server = get_server(args)
-    if server is None:
-        output.error("Unknown server '%s'", args.server_name)
-        output.close_and_exit()
+
     # Retrieves the backup
-    backup = parse_backup_id(server, args)
-    if backup is None:
-        output.error("Unknown backup '%s' for server '%s'", args.backup_id,
-                     args.server_name)
+    backup_id = parse_backup_id(server, args)
+    try:
+        for line in backup_id.get_list_of_files(args.target):
+            output.info(line, log=False)
+    except BadXlogSegmentName as e:
+        output.error(
+            "invalid xlog segment name %r\n"
+            "HINT: Please run \"barman rebuild-xlogdb %s\" "
+            "to solve this issue",
+            str(e), server.config.name)
         output.close_and_exit()
-    for line in backup.get_list_of_files(args.target):
-        output.info(line, log=False)
-    output.close_and_exit()
 
 
 @arg('server_name',
@@ -447,24 +489,120 @@ def list_files(args):
      help='specifies the backup ID')
 @expects_obj
 def delete(args):
-    """ Delete a backup
+    """
+    Delete a backup
     """
     server = get_server(args)
-    if server is None:
-        output.error("Unknown server '%s'", args.server_name)
-        output.close_and_exit()
+
     # Retrieves the backup
-    backup = parse_backup_id(server, args)
-    if backup is None:
-        output.error("Unknown backup '%s' for server '%s'", args.backup_id,
-                     args.server_name)
-        output.close_and_exit()
-    server.delete_backup(backup)
+    backup_id = parse_backup_id(server, args)
+    server.delete_backup(backup_id)
     output.close_and_exit()
 
 
+@named('get-wal')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg('wal_name',
+     help='the WAL file to get')
+@arg('--output-directory', '-o',
+     help='put the retrieved WAL file in this directory '
+          'with the original name',
+     default=SUPPRESS)
+@arg('--gzip', '-x',
+     help='compress the output with gzip',
+     action='store_const', const='gzip', dest='compression', default=SUPPRESS)
+@arg('--bzip2', '-j',
+     help='compress the output with bzip2',
+     action='store_const', const='bzip2', dest='compression', default=SUPPRESS)
+@expects_obj
+def get_wal(args):
+    """
+    Retrieve WAL_NAME file from SERVER_NAME archive.
+    The content will be streamed on standard output unless
+    the --output-directory option is specified.
+    """
+    server = get_server(args)
+    # Retrieve optional arguments. If an argument is not specified,
+    # the namespace doesn't contain it due to SUPPRESS default.
+    # In that case we pick 'None' using getattr third argument.
+    compression = getattr(args, 'compression', None)
+    output_directory = getattr(args, 'output_directory', None)
+    server.get_wal(args.wal_name,
+                   compression=compression,
+                   output_directory=output_directory)
+    output.close_and_exit()
+
+
+@named('archive-wal')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@expects_obj
+def archive_wal(args):
+    """
+    Execute maintenance operations on WAL files for a given server.
+    This command processes any incoming WAL files for the server
+    and archives them along the catalogue.
+
+    """
+    server = get_server(args)
+    server.archive_wal()
+    output.close_and_exit()
+
+
+@named('receive-wal')
+@arg('--stop', help='stop the receive-wal subprocess for the server',
+     action='store_true')
+@arg('--reset', help='reset the status of receive-wal removing '
+                     'any status files',
+     action='store_true')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@expects_obj
+def receive_wal(args):
+    """
+    Start a receive-wal process.
+    The process uses the streaming protocol to receive WAL files
+    from the PostgreSQL server.
+    """
+    server = get_server(args)
+    if args.stop and args.reset:
+        output.error("--stop and --reset options are not compatible")
+    # If the caller requested to shutdown the receive-wal process deliver the
+    # termination signal, otherwise attempt to start it
+    elif args.stop:
+        server.kill('receive-wal')
+    else:
+        server.receive_wal(reset=args.reset)
+    output.close_and_exit()
+
+
+def pretty_args(args):
+    """
+    Prettify the given argh namespace to be human readable
+
+    :type args: argh.dispatching.ArghNamespace
+    :return: the human readable content of the namespace
+    """
+    values = dict(vars(args))
+    # Retrieve the command name with recent argh versions
+    if '_functions_stack' in values:
+        values['command'] = values['_functions_stack'][0].__name__
+        del values['_functions_stack']
+    # Older argh versions only have the matching function in the namespace
+    elif 'function' in values:
+        values['command'] = values['function'].__name__
+        del values['function']
+    return "%r" % values
+
+
 def global_config(args):
-    """ Set the configuration file """
+    """
+    Set the configuration file
+    """
     if hasattr(args, 'config'):
         filename = args.config
     else:
@@ -500,66 +638,222 @@ def global_config(args):
                                  debug=args.debug)
 
     # Load additional configuration files
-    _logger.debug('Loading additional configuration files')
     config.load_configuration_files_directory()
     # We must validate the configuration here in order to have
     # both output and logging configured
     config.validate_global_config()
 
-    _logger.debug('Initialized Barman version %s (config: %s)',
-                  barman.__version__, config.config_file)
+    _logger.debug('Initialised Barman version %s (config: %s, args: %s)',
+                  barman.__version__, config.config_file, pretty_args(args))
 
 
-def get_server(args):
+def get_server(args, skip_inactive=True, skip_disabled=False,
+               on_error_stop=True, suppress_error=False):
     """
-    Get a single server from the configuration
+    Get a single server retrieving its configuration (wraps get_server_list())
 
-    :param args: an argparse namespace containing a single server_name parameter
+    Returns a Server object or None if the required server is unknown and
+    on_error_stop is False.
+
+    WARNING: this function modifies the 'args' parameter
+
+    :param args: an argparse namespace containing a single
+        server_name parameter
+        WARNING: the function modifies the content of this parameter
+    :param bool skip_inactive: skip inactive servers when 'all' is required
+    :param bool skip_disabled: skip disabled servers when 'all' is required
+    :param bool on_error_stop: stop if an error is found
+    :param bool suppress_error: suppress display of errors (e.g. diagnose)
+    :rtype: barman.server.Server|None
     """
-    config = barman.__config__.get_server(args.server_name)
-    if not config:
+    # This function must to be called with in a single-server context
+    name = args.server_name
+    assert isinstance(name, str)
+
+    # The 'all' special name is forbidden in this context
+    if name == 'all':
+        output.error("You cannot use 'all' in a single server context")
+        output.close_and_exit()
+        # The following return statement will never be reached
+        # but it is here for clarity
         return None
-    return Server(config)
+
+    # Builds a list from a single given name
+    args.server_name = [name]
+
+    # Retrieve the requested server
+    servers = get_server_list(args, skip_inactive, skip_disabled,
+                              on_error_stop, suppress_error)
+
+    # The requested server has been excluded from get_server_list result
+    if len(servers) == 0:
+        output.close_and_exit()
+        # The following return statement will never be reached
+        # but it is here for clarity
+        return None
+
+    # retrieve the server object
+    server = servers[name]
+
+    # Apply standard validation control and skips
+    # the server if inactive or disabled, displaying standard
+    # error messages. If on_error_stop (default) exits
+    if not manage_server_command(server, name) and on_error_stop:
+        output.close_and_exit()
+        # The following return statement will never be reached
+        # but it is here for clarity
+        return None
+
+    # Returns the filtered server
+    return server
 
 
-def get_server_list(args=None):
+def get_server_list(args=None, skip_inactive=False, skip_disabled=False,
+                    on_error_stop=True, suppress_error=False):
     """
     Get the server list from the configuration
 
-    If args the parameter is None or arg.server_name[0] is 'all'
+    If args the parameter is None or arg.server_name is ['all']
     returns all defined servers
 
     :param args: an argparse namespace containing a list server_name parameter
+    :param bool skip_inactive: skip inactive servers when 'all' is required
+    :param bool skip_disabled: skip disabled servers when 'all' is required
+    :param bool on_error_stop: stop if an error is found
+    :param bool suppress_error: suppress display of errors (e.g. diagnose)
+    :rtype: dict(str,barman.server.Server|None)
     """
-    if args is None or args.server_name[0] == 'all':
-        return dict(
-            (conf.name, Server(conf)) for conf in barman.__config__.servers())
+    server_dict = {}
+
+    # This function must to be called with in a multiple-server context
+    assert not args or isinstance(args.server_name, list)
+
+    # Generate the list of servers (required for global errors)
+    available_servers = barman.__config__.server_names()
+
+    # Get a list of configuration errors from all the servers
+    global_error_list = barman.__config__.servers_msg_list
+
+    # Global errors have higher priority
+    if global_error_list:
+        # Output the list of global errors
+        if not suppress_error:
+            for error in global_error_list:
+                output.error(error)
+
+        # If requested, exit on first error
+        if on_error_stop:
+            output.close_and_exit()
+            # The following return statement will never be reached
+            # but it is here for clarity
+            return {}
+
+    # Handle special 'all' server cases
+    # - args is None
+    # - 'all' special name
+    if not args or 'all' in args.server_name:
+        # When 'all' is used, it must be the only specified argument
+        if args and len(args.server_name) != 1:
+            output.error("You cannot use 'all' with other server names")
+        servers = available_servers
     else:
-        server_dict = {}
-        for server in args.server_name:
-            conf = barman.__config__.get_server(server)
-            if conf == None:
-                server_dict[server] = None
-            else:
-                server_dict[server] = Server(conf)
-        return server_dict
+        servers = args.server_name
+
+    # Loop through all the requested servers
+    for server in servers:
+        conf = barman.__config__.get_server(server)
+        if conf is None:
+            # Unknown server
+            server_dict[server] = None
+        else:
+            server_object = Server(conf)
+            # Skip inactive servers, if requested
+            if skip_inactive and not server_object.config.active:
+                output.info("Skipping inactive server '%s'"
+                            % conf.name)
+                continue
+            # Skip disabled servers, if requested
+            if skip_disabled and server_object.config.disabled:
+                output.info("Skipping temporarily disabled server '%s'"
+                            % conf.name)
+                continue
+            server_dict[server] = server_object
+
+    return server_dict
+
+
+def manage_server_command(server,
+                          name=None,
+                          inactive_is_error=False,
+                          disabled_is_error=True,
+                          skip_inactive=True,
+                          skip_disabled=True):
+    """
+    Standard and consistent method for managing server errors within
+    a server command execution. By default, suggests to skip any inactive
+    and disabled server; it also emits errors for disabled servers by
+    default.
+
+    Returns True if the command has to be executed for this server.
+
+    :param barman.server.Server server: server to be checked for errors
+    :param str name: name of the server, in a multi-server command
+    :param bool inactive_is_error: treat inactive server as error
+    :param bool disabled_is_error: treat disabled server as error
+    :param bool skip_inactive: skip if inactive
+    :param bool skip_disabled: skip if disabled
+    :return: True if the command has to be executed on this server
+    :rtype: boolean
+    """
+
+    # Unknown server (skip it)
+    if not server:
+        output.error("Unknown server '%s'" % name)
+        return False
+
+    if not server.config.active:
+        # Report inactive server as error
+        if inactive_is_error:
+            output.error('Inactive server: %s' % server.config.name)
+        if skip_inactive:
+            return False
+
+    # Report disabled server as error
+    if server.config.disabled:
+        # Output all the messages as errors, and exit terminating the run.
+        if disabled_is_error:
+            for message in server.config.msg_list:
+                output.error(message)
+        if skip_disabled:
+            return False
+
+    # All ok, execute the command
+    return True
 
 
 def parse_backup_id(server, args):
     """
     Parses backup IDs including special words such as latest, oldest, etc.
 
+    Exit with error if the backup id doesn't exist.
+
     :param Server server: server object to search for the required backup
     :param args: command lien arguments namespace
-    :rtype BackupInfo,None: the decoded backup_info object
+    :rtype: BackupInfo
     """
     if args.backup_id in ('latest', 'last'):
-        backup_id = server.get_last_backup()
+        backup_id = server.get_last_backup_id()
     elif args.backup_id in ('oldest', 'first'):
-        backup_id = server.get_first_backup()
+        backup_id = server.get_first_backup_id()
     else:
         backup_id = args.backup_id
-    return server.get_backup(backup_id)
+    backup_info = server.get_backup(backup_id)
+    if backup_info is None:
+        output.error(
+            "Unknown backup '%s' for server '%s'",
+            args.backup_id, server.config.name)
+        output.close_and_exit()
+    return backup_info
 
 
 def main():
@@ -581,19 +875,22 @@ def main():
                    default=output.DEFAULT_WRITER)
     p.add_commands(
         [
-            cron,
-            list_server,
-            show_server,
-            status,
-            check,
-            diagnose,
+            archive_wal,
             backup,
-            list_backup,
-            show_backup,
-            list_files,
-            recover,
+            check,
+            cron,
             delete,
+            diagnose,
+            get_wal,
+            list_backup,
+            list_files,
+            list_server,
             rebuild_xlogdb,
+            receive_wal,
+            recover,
+            show_backup,
+            show_server,
+            status
         ]
     )
     # noinspection PyBroadException
@@ -602,7 +899,7 @@ def main():
     except KeyboardInterrupt:
         msg = "Process interrupted by user (KeyboardInterrupt)"
         output.exception(msg)
-    except Exception, e:
+    except Exception as e:
         msg = "%s\nSee log file for more details." % e
         output.exception(msg)
 
